{% extends "!layout.html" %}
{% block sidebarlogo %}
<p align="center">
  <a href="http://oss.schmizz.net/ncclient">
    <img src="_static/logo.png" width="125" height="125"/>
  </a>
</p>
{% endblock %}
